/*
 * dhcptabtocsv.c
 * v1.02
 *
 * feed a netware generated DHCP tab to stdin and this program will
 * output a comma seperated value file containing
 * IP Address, Assignment Type, Lease Time, Host Name, Last Used,
 * Lease Expiration, and MAC Address, optionally just
 * containing a specified assignment type.
 *
 * 1.00   Initial version
 * 1.01   Added vendor recognition and assignment type option
 * 1.02   Bugfixes
 * 
 * Tony Howat 06/06/03
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include "macvendors.h"

#define VERSION "v1.02"

static char *fields[] = { "IP Address Number",
                         "Assignment Type",
                         "Lease Time",
                         "Host Name",
                         "Last Used",
                         "Lease Expiration",
                         "MAC Address",
			 "Vendor",
                         NULL };

typedef enum {
  ip_address=0,
  assignment_type,
  lease_time,
  host_name,
  last_used,
  lease_expiration,
  mac_address,
  vendor,
  count
} dhcpentries;


static char *strltrim(char *str)
{
  while(*str!='\0' && isspace(*str))
    ++str;
  
  return str;
}

static char *strrtrim(char *str)
{
  char *a;

  a=str+(strlen(str)-1);


  while(isspace(*a) && a!=str)
  {
    *a='\0';
    --a;
  } 

  return str;
}

static int parse_hex_byte(char *ptr, char **newptr)
{
  char hextable[]="0123456789ABCDEF";                
  unsigned int byte=0U;

  /* get the mac vendor ID into a unsigned int */
  if(strchr(hextable,*ptr)!=NULL)
  {
    if(strchr(hextable,*(ptr+1)))
    {
      byte=byte | ((unsigned int)(strchr(hextable,*ptr)-hextable) << 4U);
      byte=byte | (unsigned int)(strchr(hextable,*(ptr+1))-hextable);
      ptr+=2;
    } else {
      byte=strchr(hextable,*ptr)-hextable;      
      ++ptr;
    }
  }

  *newptr=ptr;
  return byte;
}

int main(int argc, char **argv)
{
  char buffer[255];
  int k=0;
  int onlylevel=-1;

  setbuf(stdout,NULL);

  if(argv[1])
  {
    if(strstr(argv[1],"h"))
    {
      fprintf(stderr,
              "\ndhcptabtocsv %s compiled %s\n\n"
              "Feed a Netware generated DHCP tab to stdin and this program\n"
              "outputs comma seperated value file containing IP Address,\n"
              "Assignment Type, Lease Time, Host Name, Last Used, Lease\n"
              "Expiration, and MAC Address, optionally just containing a\n"
              "specified assignment type. Dates are converted to\n"
              "MM/DD/YY HH:MM for importation to Excel etc. This program\n"
              "also attempts to find the vendor ID in a table converted\n"
              "from OSI documentation, and adds that field to the CSV.\n\n"
              "dhcptabtocsv [-tnnn]\n\n"
              "Where -tnnn is a lease type to restrict listings to,\n"
              "ie. -t2\n\n",VERSION,__DATE__);
      exit(0);
    }
    if(strstr(argv[1],"t"))
    {
      char *c;

      c=strstr(argv[1],"t");
      ++c;
      onlylevel=atoi(c);
    }
  }

  while(k<((dhcpentries)count))
  {
    printf("\"%s\"%s",fields[k],(k==((dhcpentries)count)-1) ? "" : ",");
    ++k;
  }
  printf("\n");

  while(!feof(stdin))
  {
    int j=0;

    fgets(buffer,255,stdin);

    if(!strncmp(buffer,"[IP Address Configuration :",26))
    {      
      char *data[(dhcpentries)count] =  { NULL };

      memset(data,0,sizeof(char *)*(dhcpentries)count);

      while(strcmp(strltrim(buffer),""))
      {
        int i=0;
        char *buf;

        fgets(buffer,255,stdin);
        buf=strltrim(buffer);

        while(fields[i]!=NULL)
	{
          if(!strncmp(buf,fields[i],strlen(fields[i])))
	  {
            char *t, *a;

            t=buf+strlen(fields[i]);

            while(*t!='=' && *t!='\0')
              ++t;
            if(*t=='=') 
              ++t;
            while(isspace(*t) && *t!='\0')
              ++t;
	    strrtrim(t);

            if(data[i]==NULL)
	    {
              data[i]=(char *)malloc((strlen(t))+2);
              if(data[i])
                strcpy(data[i],t);
	    }
          }
          
          ++i;
        }

      }

      if(onlylevel==-1 | 
         (data[(dhcpentries)assignment_type] && 
	 onlylevel==atoi(data[(dhcpentries)assignment_type])))
      {
        j=0;
        while(j<((dhcpentries)count))
        {
          if(data[j])
          {
            switch(j)
	    {
	      case mac_address:
	      {
                char *ptr;
                unsigned int mac=0;

                ptr=data[j];

                if(ptr && ptr[0]!='\0')
                {
                  /* skip the bizarre "1 " prepended to MAC addresses
                   */
                  if(*ptr=='1' && *(ptr+1)==' ')
                    ptr+=2;

                  mac=parse_hex_byte(ptr,&ptr) << 16;
                  if(*ptr==':' || *ptr==' ')
                    ++ptr;
                  mac|=parse_hex_byte(ptr,&ptr) << 8;
                  if(*ptr==':' || *ptr==' ')
                    ++ptr;
                  mac|=parse_hex_byte(ptr,&ptr);
                  if(*ptr==':' || *ptr==' ')
                    ++ptr;
                }

                if(mac && data[j]!='\0')
		{
                  if(*(data[j])=='1' && *((data[j])+1)==' ')
                    printf("%s%s",((data[j])+2),((dhcpentries)j==count-1) ? "\n" : ",");
                  else
                    printf("%s%s",data[j],((dhcpentries)j==count-1) ? "\n" : ",");
                  data[(dhcpentries)vendor]=(char *)malloc(strlen(Get_Mac_Vendor(mac))+1);
	          if(data[(dhcpentries)vendor])
                  {
                    strcpy(data[(dhcpentries)vendor],Get_Mac_Vendor(mac));
                  }
                  free(data[j]); data[j]=NULL;
                } else {
                  printf("%s%s",data[j],((dhcpentries)j==count-1) ? "\n" : ",");
                  free(data[j]);
                  data[j]=NULL;
                }

                break;
              }

	      case last_used:
	      case lease_expiration:
	      {
                time_t t;
                struct tm *tmstruct;
                char outtime[32]="";

                /* divide by 1000 in the crudest way possible to avoid 
                   long longs */
                if(!strncmp(data[j]+strlen(data[j])-3,"000",3))
	        {
                  data[j][strlen(data[j])-3]='\0';
                }
  
                t=(time_t)atol(data[j]);

                tmstruct=(struct tm *)localtime(&t);

                strftime(outtime,32,"%m/%d/%y %H:%M",tmstruct);

                if(t)
                  printf("%s%s",outtime,
                         ((dhcpentries)j==count-1) ? "\n" : ",");
                else
                  printf(",%s",((dhcpentries)j==count-1) ? "" : ",");
                free(data[j]); data[j]=NULL;
                break;
              }
              case vendor:
                printf("\"%s\"%s",data[j],((dhcpentries)j==count-1) ? "\n" : ",");
                free(data[j]); data[j]=NULL;
                break;
	      default:
	      {
                printf("%s%s",data[j],((dhcpentries)j==count-1) ? "\n" : ",");
                free(data[j]); data[j]=NULL;
                break;
              }
            }
          } else {
            if(j!=(dhcpentries)count-1)
              printf(",");
            else
              printf("\n");
          }
          ++j;
        }
      }
    }
  }
}
